searchNodes=[{"doc":"This module contains helper methods for loading and using the PKCS #11 module for NervesKey in Elixir. You don't need to use these methods to use the shared library.","ref":"NervesKey.PKCS11.html","title":"NervesKey.PKCS11","type":"module"},{"doc":"Load the OpenSSL engine","ref":"NervesKey.PKCS11.html#load_engine/0","title":"NervesKey.PKCS11.load_engine/0","type":"function"},{"doc":"Return the key map for passing a private key to ssl_opts.This method creates the key map that the :crypto library can use to properly route private key operations to the PKCS #11 shared library.Options::i2c - which I2C bus (defaults to I2C bus 0 (/dev/i2c-0)):certificate - which certificate on the NervesKey to use (defaults to :primary)Passing {:i2c, 1} is still supported, but should be updated to use keyword list form for the options.","ref":"NervesKey.PKCS11.html#private_key/2","title":"NervesKey.PKCS11.private_key/2","type":"function"},{"doc":"The device/signer certificate pair to use","ref":"NervesKey.PKCS11.html#t:certificate_pair/0","title":"NervesKey.PKCS11.certificate_pair/0","type":"type"},{"doc":"I2C bus","ref":"NervesKey.PKCS11.html#t:i2c_bus/0","title":"NervesKey.PKCS11.i2c_bus/0","type":"type"},{"doc":"Option for which NervesKey and certificate to use.:i2c - which I2C bus:certificate - which NervesKey certificate to use (:primary or :aux)","ref":"NervesKey.PKCS11.html#t:option/0","title":"NervesKey.PKCS11.option/0","type":"type"},{"doc":"NervesKey.PKCS11 This is a minimal implementation of PKCS #11 for using the NervesKey with OpenSSL and other programs. The NervesKey is a specific configuration of the ATECC508A/ATECC608A chips that holds one private key in slot 0. If you're using this chip in a similar configuration, this should work for you as well.Supported features:ECDSAThis library is organized to make it easy to integrate into Elixir and is written with an expectation that provisioning, extracting certificates, etc. is done via other means (like using nerves_key.) If you're not using Elixir, you can still run make and copy priv/nerves_key_pkcs11.so to a conveniently location. Elixir isn't needed to build the C library.Another option is to look at cryptoauth-openssl-engine or cryptoauthlib.","ref":"readme.html","title":"NervesKey.PKCS11","type":"extras"},{"doc":"This library is self-contained with no compile-time dependencies. At runtime, it isn't useful unless you have another program around that uses a PKCS #11 shared library.If using it with Elixir, add a dependency to your mix.exs:def deps do [ {:nerves_key_pkcs11, &quot;~&gt; 0.2.2&quot;} ] endIf not using Elixir, run make. You may need to set $(CC) or $(CFLAGS) if you're crosscompiling.","ref":"readme.html#building","title":"NervesKey.PKCS11 - Building","type":"extras"},{"doc":"PKCS #11 uses the term slot to refer to cryptographic devices. This library can use either slot ID (if called directly) or the slot's token ID (if called via libp11) to find the NervesKey. Various parameters are mapped into the slot ID according to the table below:Slot range | I2C bus | Bus address | Certificate ------------|-----------|-------------|------------ 0-15 | slot | 0x60 | Primary or auxiliaryOn Linux, the I2C bus number in the table above determines the device file. For example, a bus number of &quot;1&quot; maps to /dev/i2c-1. NervesKey devices support a primary and auxiliary set of certificates. Normally only the primary device certificate is used. Under some conditions, it's useful to write a second set of certificates to the device and those can be referenced by using the &quot;Auxiliary&quot; rows in the table. This library currently need to differentiate between primary and auxiliary certificates so that's not represented in the slot ID. It may be in the future.The PKCS #11 URI for addressing the desired NervesKey has the form:pkcs11:token=1","ref":"readme.html#slot-definition","title":"NervesKey.PKCS11 - Slot definition","type":"extras"},{"doc":"To use this with OpenSSL, you'll need libpkcs11.so. This library comes from OpenSC's libp11 and can be installed on Debian systems by running:sudo apt install libengine-pkcs11-openssl1.1","ref":"readme.html#openssl-integration","title":"NervesKey.PKCS11 - OpenSSL integration","type":"extras"},{"doc":"Erlang's crypto application provides an API for loading OpenSSL engines. See the Erlang crypto User's Guide for details on this feature. NervesKey.PKCS11.load_engine/0 is a helper method to make the :crypto.engine_load/3 call for you. It uses OpenSSL's dynamic engine to load libpkcs11.so which in turn loads this PKCS #11 implementation. Here's an example call in Elixir:{:ok, engine} = NervesKey.PKCS11.load_engine()If this doesn't work, you'll likely have to look at the implementation of load_engine/0 and fine tune the shared library paths or control commands.After you load the engine, you'll eventually want to use it. The intended use case is for delegate the ECDSA operation to the ATECC508A for use with TLS connections. You'll need to obtain the X.509 certificate that corresponds to the private key held in the ATECC508A through some mechanism. Then in your SSL options, you'll have something like this:[ key: NervesKey.PKCS11.private_key(engine, i2c: 1), certfile: &quot;device-cert.pem&quot;, ]The NervesKey.PKCS11.private_key/2 helper method will create the appropriate map so that Erlang's :crypto library can properly call into OpenSSL.","ref":"readme.html#invocation-from-erlang-and-elixir","title":"NervesKey.PKCS11 - Invocation from Erlang and Elixir","type":"extras"},{"doc":"If you have other code using the NervesKey, it might conflict with this library. There's no lock file or mechanism to keep more than one process from accessing the ATECC508A chip simultaneously. This is not expected to be an issue at runtime since the main reason to access the NervesKey in another process is to provision it and that's not something one would do when trying to use this library to assist a TLS negotiation.","ref":"readme.html#sharing-the-nerveskey","title":"NervesKey.PKCS11 - Sharing the NervesKey","type":"extras"},{"doc":"The Elixir and most C code is licensed under the 2-Clause BSD License.The header file for the PKCS #11 function prototypes and structures, pkcs11.h, has the following license:/* pkcs11.h Copyright 2006, 2007 g10 Code GmbH Copyright 2006 Andreas Jellinghaus This file is free software; as a special exception the author gives unlimited permission to copy and/or distribute it, with or without modifications, as long as this notice is preserved. This file is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY, to the extent permitted by law; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. */","ref":"readme.html#license","title":"NervesKey.PKCS11 - License","type":"extras"}]